import React, { useState, useRef, useEffect, useMemo, useCallback } from "react";
import { useNavigate } from 'react-router-dom';
import request from '@/utils/request';

import {
    List,
    DatagridConfigurable,
    SearchInput,
    TopToolbar,
    SelectColumnsButton,
    FilterButton,
    BulkDeleteButton,
    WrapperField,
    useRecordContext,
    useTranslate,
    useNotify,
    useRefresh,
    TextField,
    NumberField,
    DateField,
    BooleanField,
    TextInput,
    DateInput,
    SelectInput,
    NumberInput,

    Button,
} from 'react-admin';
import { Box, Typography, Card, Stack, Drawer } from '@mui/material';
import { styled } from '@mui/material/styles';
import TaskCreate from "./TaskCreate";
import TaskPanel from "./TaskPanel";
import EmptyData from "../components/EmptyData";
import MyCreateButton from "../components/MyCreateButton";
import MyExportButton from '../components/MyExportButton';
import SwapVertIcon from '@mui/icons-material/SwapVert';
import AlignVerticalTopIcon from '@mui/icons-material/AlignVerticalTop';
import TaskAltIcon from '@mui/icons-material/TaskAlt';
import CancelIcon from '@mui/icons-material/Cancel';
import PageDrawer from "../components/PageDrawer";
import MyField from "../components/MyField";
import { PAGE_DRAWER_WIDTH, OPERATE_MODE, DEFAULT_PAGE_SIZE } from '@/config/setting';
import * as Common from '@/utils/common';

const StyledDatagrid = styled(DatagridConfigurable)(({ theme }) => ({
    '& .css-1vooibu-MuiSvgIcon-root': {
        height: '.9em'
    },
    '& .RaDatagrid-row': {
        cursor: 'auto'
    },
    '& .column-name': {
    },
    '& .opt': {
        width: 248
    },
}));

const filters = [
    <SearchInput source="condition" alwaysOn />,
    <DateInput label='common.time.after' source="timeStart" />,
    <DateInput label='common.time.before' source="timeEnd" />,
    <TextInput source="taskCode" label="table.field.task.taskCode" />,
    <NumberInput source="taskStatus" label="table.field.task.taskStatus" />,
    <NumberInput source="taskType" label="table.field.task.taskType" />,
    <TextInput source="orgLoc" label="table.field.task.orgLoc" />,
    <TextInput source="orgSite" label="table.field.task.orgSite" />,
    <TextInput source="targLoc" label="table.field.task.targLoc" />,
    <TextInput source="targSite" label="table.field.task.targSite" />,
    <TextInput source="barcode" label="table.field.task.barcode" />,
    <TextInput source="robotCode" label="table.field.task.robotCode" />,
    <NumberInput source="exceStatus" label="table.field.task.exceStatus" />,
    <TextInput source="expDesc" label="table.field.task.expDesc" />,
    <NumberInput source="sort" label="table.field.task.sort" />,
    <TextInput source="expCode" label="table.field.task.expCode" />,
    <DateInput source="startTime" label="table.field.task.startTime" />,
    <DateInput source="endTime" label="table.field.task.endTime" />,
    <TextInput label="common.field.memo" source="memo" />,
    <SelectInput
        label="common.field.status"
        source="status"
        choices={[
            { id: '1', name: 'common.enums.statusTrue' },
            { id: '0', name: 'common.enums.statusFalse' },
        ]}
        resettable
    />,
]

const TaskList = () => {
    const translate = useTranslate();
    const [drawerVal, setDrawerVal] = useState(false);
    return (
        <Box display="flex">
            <List
                sx={{
                    flexGrow: 1,
                    transition: (theme) =>
                        theme.transitions.create(['all'], {
                            duration: theme.transitions.duration.enteringScreen,
                        }),
                    marginRight: !!drawerVal ? `${PAGE_DRAWER_WIDTH}px` : 0,
                }}
                title={"menu.task"}
                empty={false}
                filters={filters}
                sort={{ field: "sort", order: "desc" }}
                actions={(
                    <TopToolbar>
                        <FilterButton />
                        <SelectColumnsButton preferenceKey='task' />
                    </TopToolbar>
                )}
                perPage={DEFAULT_PAGE_SIZE}
            >
                <StyledDatagrid
                    preferenceKey='task'
                    bulkActionButtons={
                        <>
                            <BulkResortButton />
                            <BulkCancelButton />
                            <BulkDeleteButton mutationMode={OPERATE_MODE} />
                        </>
                    }
                    rowClick={'edit'}
                    expand={false}
                    expandSingle={true}
                    omit={['id', 'createTime', 'createBy', 'memo', 'robotCode', 'exceStatus', 'expDesc', 'expCode', 'status', 'sort']}
                >
                    <NumberField source="id" />
                    <TextField source="taskCode" label="table.field.task.taskCode" />
                    <NumberField source="taskStatus$" label="table.field.task.taskStatus" />
                    <NumberField source="taskType$" label="table.field.task.taskType" />
                    <TextField source="orgLoc" label="table.field.task.orgLoc" />
                    <TextField source="orgSite" label="table.field.task.orgSite" />
                    <TextField source="targLoc" label="table.field.task.targLoc" />
                    <TextField source="targSite" label="table.field.task.targSite" />
                    <TextField source="barcode" label="table.field.task.barcode" />
                    <TextField source="robotCode" label="table.field.task.robotCode" />
                    <NumberField source="exceStatus" label="table.field.task.exceStatus" />
                    <TextField source="expDesc" label="table.field.task.expDesc" />
                    <NumberField source="sort" label="table.field.task.sort" />
                    <TextField source="expCode" label="table.field.task.expCode" />
                    <TextField source="updateBy$" label="common.field.updateBy"/>
                    <TextField source="createBy$" label="common.field.createBy"/>
                    <DateField source="updateTime" label="common.field.updateTime" showTime />
                    <DateField source="createTime" label="common.field.createTime" showTime />
                    {/* <BooleanField source="statusBool" label="common.field.status" sortable={false} /> */}
                    <TextField source="memo" label="common.field.memo" sortable={false} />
                    <WrapperField cellClassName="opt" label="common.field.opt">
                        <DoneButton sx={{ padding: '1px', fontSize: '.75rem' }} ></DoneButton>
                        <CancelButton sx={{ padding: '1px', fontSize: '.75rem' }} mutationMode={OPERATE_MODE} />
                        <SetTopButton sx={{ padding: '1px', fontSize: '.75rem' }} ></SetTopButton>
                    </WrapperField>
                </StyledDatagrid>
            </List>
            <PageDrawer
                title='Task Detail'
                drawerVal={drawerVal}
                setDrawerVal={setDrawerVal}
            >
            </PageDrawer>
        </Box>
    )
}

export default TaskList;


/**
 * 完成操作
 * @returns 
 */
const DoneButton = () => {
    const record = useRecordContext();
    const clickComplete = () => {
        completeTask([record])
    };
    //完成任务
    const completeTask = async (row) => { }
    return (
        <Button
            onClick={clickComplete}
            label="toolbar.complete">
            <TaskAltIcon />
        </Button>
    )
}

/**
 * 取消按钮
 * @returns 
 */
const CancelButton = () => {
    const record = useRecordContext();
    const notify = useNotify();
    const refresh = useRefresh();
    const clickCancel = () => {
        cancleTask(record)
    };
    //取消任务
    const cancleTask = async (row) => {
        console.log(row);
        const { data: { code, data, msg } } = await request.post(`/task/remove/` + row.id);
        if (code === 200) {
            notify(msg);
            refresh();
        } else {
            notify(msg);
        }
    }
    return (
        <Button
            onClick={clickCancel}
            label="toolbar.cancel">
            <CancelIcon />
        </Button>
    )
}
/**
 *  置顶操作
 * @returns 
 */
const SetTopButton = () => {
    const record = useRecordContext();
    const notify = useNotify();
    const refresh = useRefresh();
    const clickTop = () => {
        topTask(record)
    };
    //置顶任务
    const topTask = async (row) => {
        const { data: { code, data, msg } } = await request.post(`/task/top/` + row.id);
        if (code === 200) {
            notify(msg);
            refresh();
        } else {
            notify(msg);
        }
     }
    return (
        <Button
            onClick={clickTop}
            label="toolbar.top">
            <AlignVerticalTopIcon />
        </Button>
    )
}

/**
 * 批量取消
 * @returns 
 */
const BulkCancelButton = () => {
    const record = useRecordContext();
    const clickCancel = () => {
        cancleTask([record])
    };
    //取消任务
    const cancleTask = async (row) => { }
    return (
        <Button
            onClick={clickCancel}
            label="toolbar.cancel">
            <CancelIcon />
        </Button>
    )
}

/**
 * 批量排序
 * @returns 
 */
const BulkResortButton = () => {
    const record = useRecordContext();
    const bulkResort = () => {
        resortTask([record])
    };
    //批量排序
    const resortTask = async (row) => { }
    return (
        <Button
            onClick={bulkResort}
            label="toolbar.resort">
            <SwapVertIcon />
        </Button>
    )
}